
/***************************************************************************

                                 ChessV

                   COPYRIGHT (C) 2005 BY GREGORY STRONG

This file is part of ChessV.  ChessV is free software; you can redistribute
it and/or modify it under the terms of the GNU General Public License as 
published by the Free Software Foundation; either version 2 of the License, 
or (at your option) any later version.

ChessV is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
more details; the file 'COPYING' contains the License text, but if for
some reason you need a copy, please write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

****************************************************************************/


#include "StdAfx.h"
#include "ChessV.h"
#include "PieceType.h"
#include "Piece.h"
#include "Direction.h"
#include "ChessGame.h"
#include "Rand.h"
#include "GameParameters.h"


#include "Chess_Data.h"
//	this include file contains data matricies for square bonuses,
//	outpost bonuses, etc.  it also contains some macros used here.


void StartChessGame()
{
	int rtn = 0;
	if( !justLoaded )
	{
		if( !strcmp( gameSelection, "Orthodox Chess" ) )
		{
			rtn = (int) ::DialogBox( theInstance, MAKEINTRESOURCE(IDD_CHESS_SELECT_DIALOG), NULL, 
				reinterpret_cast<DLGPROC>(Chess_SelectDlgProc) );
		}
		else if( !strcmp( gameSelection, "Chess with Different Armies" ) )
		{
			rtn = (int) ::DialogBox( theInstance, MAKEINTRESOURCE(IDD_CWDA_SELECT_DIALOG), NULL, 
				reinterpret_cast<DLGPROC>(CWDA_SelectDlgProc) );
		}
	}

	theBoard = new Board( 8, 8 );
	CreatePlayers( "White", "Black", whiteComp, blackComp );
}

Game *CreateChessGame( Board &brd, Player &plr0, Player &plr1 )
{
	return new ChessGame( brd, plr0, plr1 );
}

ChessGame::ChessGame( Board &board, Player &whitePlayer, Player &blackPlayer ):
	Game(board, whitePlayer, blackPlayer),
	randomSetup(false),
	chessGameDerived(false),
	enPassant(true)
{
	//	turn on use of pawn structure evaluation
	usePawnStructureEvaluation = true;

	//	implement 50-move draw rule
	autoDrawPeriod = 100;

	//	turn off null-move and futility pruning after 24 pieces 
	//	have been captured.  when the number of pieces remaining is 
	//	low, we risk a zugzwang situation, and trying null moves in 
	//	such situations is a recipe for disaster.
	endgameCaptureThreshold = 24;

	//	razoring and futility pruning margins
	razorMargin = 6500;
	futilityMargin = 2800;
	extendedFutilityMargin = 3800;

	//	activate Bit Boards for supported games
	if( !strcmp( selectedVariant->name, "Orthodox Chess" ) || 
		!strcmp( selectedVariant->name, "Fisher Random Chess" ) || 
		!strcmp( selectedVariant->name, "Chess 480" ) )
	{
		customMoveGeneration = true;
		customCaptureGeneration = true;
	}


	// *** PHASES *** //

	//	we have to set up the phases we wish to have in this game;
	//	we will keep it simple and have 3: opening, midgame, and endgame
	nPhases = 3;

	//	settings for phase 0 (opening)
	phases[0].SetNumber( 1 );
	phases[0].SetMobilityFactor( 1 );
	phases[0].SetPawnDeficiencyFactor( 8 );
	phases[0].SetSquareValuesFactor( 1 );
	phases[0].SetKingSafetyFactor( 4 );
	phases[0].SetTropismFactor( 6 );

	//	settings for phase 1 (midgame)
	phases[1].SetNumber( 1 );
	phases[1].SetMobilityFactor( 1 );
	phases[1].SetPawnDeficiencyFactor( 10 );
	phases[1].SetSquareValuesFactor( 1 );
	phases[1].SetKingSafetyFactor( 8 );
	phases[1].SetTropismFactor( 10 );
	
	//	settings for phase 2 (endgame)
	phases[2].SetNumber( 2 );
	phases[2].SetMobilityFactor( 0 );
	phases[2].SetPawnDeficiencyFactor( 16 );
	phases[2].SetSquareValuesFactor( 1 );
	phases[2].SetKingSafetyFactor( 0 );
	phases[2].SetTropismFactor( 12 );


	// *** OUTPOSTS *** //

	//	we wish to be able to give bonuses to pieces that are 'posted'-
	//	that is, which are centrally located and cannot be reached by
	//	enemy pawns (because adjacent pawns are gone, or have advanced
	//	beyond the piece).  we also wish to be able to give different
	//	bonuses for different types of minor pieces (a posted Knight is
	//	worth more than a posted Bishop.)

	//	we will use the user variable 'gameInt1' in the PieceType class
	//	to store the 'outpost factor' for a given piece.  this value will
	//	be multiplied by the square-values stored in the 'outpost' array.
	OrthodoxKnight::orthodoxKnight.gameInt1 = 10;
	OrthodoxBishop::orthodoxBishop.gameInt1 = 7;
	Waffle::waffle.gameInt1 = 5;
	Fad::fad.gameInt1 = 9;
	ChargingKnight::chargingKnight.gameInt1 = 10;
	Fibnif::fibnif.gameInt1 = 8;
	WoodyRook::woodyRook.gameInt1 = 5;
	HalfDuck::halfDuck.gameInt1 = 9;
	Knishop::knishop.gameInt1 = 9;
	Bishight::bishight.gameInt1 = 9;
	B4nD::b4nD.gameInt1 = 10;
	SV7::sv7.gameInt1 = 9;
	Cyborg::cyborg.gameInt1 = 9;
	

	// *** INITIALIZATION *** //

	for( int x = 0; x < 8; x++ )
	{
		player0PawnTwoSpaceZone.SetBit( x + 8 );
		player1PawnTwoSpaceZone.SetBit( x + 48 );
	}

	//	initialize hashes
	enPassantHashMap = new HashMap( board.GetNumberOfSquares() );
	castlingHash0k = rand_32();
	castlingHash0q = rand_32();
	castlingHash1k = rand_32();
	castlingHash1q = rand_32();

	//	some defaults
	castlingPiecesAreColorbound[0] = false;
	castlingPiecesAreColorbound[1] = false;

	//	basic initialization
	board.Initialize( this, BITBOARD_64 );
}

void ChessGame::AddPlayerPieceTypes()
{
	//	add the piece types that each player may have
	board.AddPlayerPieceTypeBothPlayers( OrthodoxRook::orthodoxRook );
	board.AddPlayerPieceTypeBothPlayers( OrthodoxBishop::orthodoxBishop );
	board.AddPlayerPieceTypeBothPlayers( OrthodoxKnight::orthodoxKnight);
	board.AddPlayerPieceTypeBothPlayers( OrthodoxQueen::orthodoxQueen );
	board.AddPlayerPieceTypeBothPlayers( OrthodoxKing::orthodoxKing );
	board.AddPlayerPieceTypeBothPlayers( OrthodoxPawn::orthodoxPawn );
	board.AddPlayerPieceTypeBothPlayers( Chancellor::chancellor );
	board.AddPlayerPieceTypeBothPlayers( Cardinal::cardinal );
}

void ChessGame::InitializeBoard()
{
	//	add the piece types that each player may have
	AddPlayerPieceTypes();

	//	initialize piece type references
	castlingPieceTypes[0] = &OrthodoxRook::orthodoxRook;
	castlingPieceTypes[1] = &OrthodoxRook::orthodoxRook;
	minorPieceTypes[0][0] = &OrthodoxKnight::orthodoxKnight;
	minorPieceTypes[1][0] = &OrthodoxKnight::orthodoxKnight;
	minorPieceTypes[0][1] = &OrthodoxBishop::orthodoxBishop;
	minorPieceTypes[1][1] = &OrthodoxBishop::orthodoxBishop;
	minorPieceTypes[0][2] = &NullPieceType64::nullPieceType64;
	minorPieceTypes[1][2] = &NullPieceType64::nullPieceType64;
	minorPieceTypes[0][3] = &NullPieceType64::nullPieceType64;
	minorPieceTypes[1][3] = &NullPieceType64::nullPieceType64;
	colorboundPieceTypes[0][0] = &OrthodoxBishop::orthodoxBishop;
	colorboundPieceTypes[1][0] = &OrthodoxBishop::orthodoxBishop;
	colorboundPieceTypes[0][1] = &NullPieceType64::nullPieceType64;
	colorboundPieceTypes[1][1] = &NullPieceType64::nullPieceType64;
	queenPieceTypes[0] = &OrthodoxQueen::orthodoxQueen;
	queenPieceTypes[1] = &OrthodoxQueen::orthodoxQueen;
	pawnPieceTypes[0] = &OrthodoxPawn::orthodoxPawn;
	pawnPieceTypes[1] = &OrthodoxPawn::orthodoxPawn;
	castlingPiecesSlideNorth[0] = true;
	castlingPiecesSlideNorth[1] = true;

	castlingType = CASTLING_TYPE_STANDARD;

	//	if we aren't playing one of the games directly supported by this class, 
	//	then we are playing a game from a ChessGame-derived class, so we will 
	//	return here, and leave the setup to that class.
	if( strcmp( selectedVariant->name, "Orthodox Chess" ) &&
		strcmp( selectedVariant->name, "Almost Chess" ) &&
		strcmp( selectedVariant->name, "Fisher Random Chess" ) &&
		strcmp( selectedVariant->name, "Chess with Different Armies" ) &&
		strcmp( selectedVariant->name, "Chess with Augmented Knights" ) )
	{
		chessGameDerived = true;
		return;
	}

	int rook1File = 0;   // \ 
	int rook2File = 7;   //  \  
	int knight1File = 1; //   \    these are the default files on which each 
	int knight2File = 6; //    >-  type of piece starts.  we define these so
	int bishop1File = 2; //   /    that we can shuffle them around if we are
	int bishop2File = 5; //  /     playing Fisher Random Chess.
	int queenFile = 3;   // /
	int kingFile = 4;


	// *** FISHER RANDOM CHESS *** //
	if( !strcmp( selectedVariant->name, "Fisher Random Chess" ) || 
		!strcmp( selectedVariant->name, "Chess 480" ) )
	{
		randomSetup = true;
		LookupIntParameter( "rook1File", rook1File );
		LookupIntParameter( "rook2File", rook2File );
		LookupIntParameter( "bishop1File", bishop1File );
		LookupIntParameter( "bishop2File", bishop2File );
		LookupIntParameter( "knight1File", knight1File );
		LookupIntParameter( "knight2File", knight2File );
		LookupIntParameter( "queenFile", queenFile );
		LookupIntParameter( "kingFile", kingFile );

		if( !strcmp( selectedVariant->name, "Fisher Random Chess" ) )
			castlingType = CASTLING_TYPE_FISCHER_RANDOM;
		else
			castlingType = CASTLING_TYPE_CHESS480;

		//	no opening book support for Fisher Random Chess
		//	because there are 960 possible setups
		useOpeningBook = false;
	}

	//	add the kings; all variants supported by this class have Kings
	AddPiece( OrthodoxKing::orthodoxKing, 0, 0, kingFile );
	AddPiece( OrthodoxKing::orthodoxKing, 1, 7, kingFile );

	//	add the pawns
	for( int i = 0; i < 8; i++ )
	{
		//	for all variants supported by this class, we will have these
		//	8 pawns lined up on the 2nd and 7th rank
		AddPiece( OrthodoxPawn::orthodoxPawn, 0, 1, i );
		AddPiece( OrthodoxPawn::orthodoxPawn, 1, 6, i );
	}


	// *** CHESS and CHESS WITH DIFFERENT ARMIES *** //

	//	piece together the name of the opening book for 
	//	this particular combination of armies
	char filename[] = "openings\\CWDA_FaF_v_FaF.txt";

	if( selectedVariant->resourceID == IDC_CWDA_RADIO ||
		selectedVariant->resourceID == IDC_CHESS_RADIO )
	{
		char *armyName;
		if( LookupStringParameter( "white-army", armyName ) )
		{
			if( !strcmp( selectedVariant->name, "Orthodox Chess" ) || 
				!strcmp( selectedVariant->name, "Fisher Random Chess" ) || 
				!strcmp( selectedVariant->name, "Chess 480" ) || 
				!strcmp( selectedVariant->name, "Almost Chess" ) || 
				!strcmp( armyName, "Fabulous FIDEs" ) ||
				!strcmp( armyName, "Almost Chess army" ) )
			{
				AddPiece( OrthodoxRook::orthodoxRook, 0, 0, rook1File );
				AddPiece( OrthodoxRook::orthodoxRook, 0, 0, rook2File );
				AddPiece( OrthodoxKnight::orthodoxKnight, 0, 0, knight1File );
				AddPiece( OrthodoxKnight::orthodoxKnight, 0, 0, knight2File );
				AddPiece( OrthodoxBishop::orthodoxBishop, 0, 0, bishop1File );
				AddPiece( OrthodoxBishop::orthodoxBishop, 0, 0, bishop2File );
				if( armyName != NULL && !strcmp( armyName, "Almost Chess army" ) )
				{
					filename[14] = 'A';
					filename[15] = 'l';
					filename[16] = 'C';
					AddPiece( Chancellor::chancellor, 0, 0, queenFile );
				}
				else
					AddPiece( OrthodoxQueen::orthodoxQueen, 0, 0, queenFile );
			}
			else if( !strcmp( armyName, "Colorbound Clobberers" ) )
			{
				filename[14] = 'C';
				filename[15] = 'b';
				filename[16] = 'C';
				AddPiece( Bede::bede, 0, 0, rook1File );
				AddPiece( Bede::bede, 0, 0, rook2File );
				AddPiece( Waffle::waffle, 0, 0, knight1File );
				AddPiece( Waffle::waffle, 0, 0, knight2File );
				AddPiece( Fad::fad, 0, 0, bishop1File );
				AddPiece( Fad::fad, 0, 0, bishop2File );
				AddPiece( Cardinal::cardinal, 0, 0, queenFile );
				castlingPiecesSlideNorth[0] = false;
			}
			else if( !strcmp( armyName, "Colorbound Clobberers II" ) )
			{
				filename[14] = 'C';
				filename[15] = 'C';
				filename[16] = '2';
				AddPiece( Bede::bede, 0, 0, rook1File );
				AddPiece( Bede::bede, 0, 0, rook2File );
				AddPiece( Waffle::waffle, 0, 0, bishop1File );
				AddPiece( Waffle::waffle, 0, 0, bishop2File );
				AddPiece( Fad::fad, 0, 0, knight1File );
				AddPiece( Fad::fad, 0, 0, knight2File );
				AddPiece( Cardinal::cardinal, 0, 0, queenFile );
				castlingPiecesSlideNorth[0] = false;
			}
			else if( !strcmp( armyName, "Nutty Knights" ) )
			{
				filename[14] = 'N';
				filename[15] = 'K';
				filename[16] = 'n';
				AddPiece( ChargingRook::chargingRook, 0, 0, rook1File );
				AddPiece( ChargingRook::chargingRook, 0, 0, rook2File );
				AddPiece( Fibnif::fibnif, 0, 0, knight1File );
				AddPiece( Fibnif::fibnif, 0, 0, knight2File );
				AddPiece( ChargingKnight::chargingKnight, 0, 0, bishop1File );
				AddPiece( ChargingKnight::chargingKnight, 0, 0, bishop2File );
				AddPiece( Colonel::colonel, 0, 0, queenFile );
			}
			else if( !strcmp( armyName, "Remarkable Rookies" ) )
			{
				filename[14] = 'R';
				filename[15] = 'R';
				filename[16] = 'k';
				AddPiece( ShortRook::shortRook, 0, 0, rook1File );
				AddPiece( ShortRook::shortRook, 0, 0, rook2File );
				AddPiece( WoodyRook::woodyRook, 0, 0, knight1File );
				AddPiece( WoodyRook::woodyRook, 0, 0, knight2File );
				AddPiece( HalfDuck::halfDuck, 0, 0, bishop1File );
				AddPiece( HalfDuck::halfDuck, 0, 0, bishop2File );
				AddPiece( Chancellor::chancellor, 0, 0, queenFile );
			}
			else if( !strcmp( armyName, "Forward FIDEs" ) )
			{
				filename[14] = 'F';
				filename[15] = 'w';
				filename[16] = 'F';
				AddPiece( ChargingRook::chargingRook, 0, 0, rook1File );
				AddPiece( ChargingRook::chargingRook, 0, 0, rook2File );
				AddPiece( Knishop::knishop, 0, 0, knight1File );
				AddPiece( Knishop::knishop, 0, 0, knight2File );
				AddPiece( Bishight::bishight, 0, 0, bishop1File );
				AddPiece( Bishight::bishight, 0, 0, bishop2File );
				AddPiece( Forequeen::forequeen, 0, 0, queenFile );
			}
			else if( !strcmp( armyName, "All-Around Allstars" ) )
			{
				filename[14] = 'A';
				filename[15] = 'A';
				filename[16] = 's';
				AddPiece( ChargingRook::chargingRook, 0, 0, rook1File );
				AddPiece( ChargingRook::chargingRook, 0, 0, rook2File );
				AddPiece( Waffle::waffle, 0, 0, knight1File );
				AddPiece( Waffle::waffle, 0, 0, knight2File );
				AddPiece( OrthodoxBishop::orthodoxBishop, 0, 0, bishop1File );
				AddPiece( OrthodoxBishop::orthodoxBishop, 0, 0, bishop2File );
				AddPiece( Chancellor::chancellor, 0, 0, queenFile );
			}
			else if( !strcmp( armyName, "Meticulous Mashers" ) )
			{
				filename[14] = 'M';
				filename[15] = 'M';
				filename[16] = 'a';
				AddPiece( Fourfer::fourfer, 0, 0, rook1File );
				AddPiece( Fourfer::fourfer, 0, 0, rook2File );
				AddPiece( Fibnif::fibnif, 0, 0, knight1File );
				AddPiece( Fibnif::fibnif, 0, 0, knight2File );
				AddPiece( B4nD::b4nD, 0, 0, bishop1File );
				AddPiece( B4nD::b4nD, 0, 0, bishop2File );
				AddPiece( N2R4::n2R4, 0, 0, queenFile );
			}
			else if( !strcmp( armyName, "Andromedan Invaders" ) )
			{
				filename[14] = 'A';
				filename[15] = 'n';
				filename[16] = 'I';
				AddPiece( SV7::sv7, 0, 0, rook1File );
				AddPiece( SV7::sv7, 0, 0, rook2File );
				AddPiece( GreaterKnight::greaterKnight, 0, 0, knight1File );
				AddPiece( GreaterKnight::greaterKnight, 0, 0, knight2File );
				AddPiece( Cyborg::cyborg, 0, 0, bishop1File );
				AddPiece( Cyborg::cyborg, 0, 0, bishop2File );
				AddPiece( ShortQueen::shortQueen, 0, 0, queenFile );
			}
		}
		if( LookupStringParameter( "black-army", armyName ) )		{
			if( !strcmp( selectedVariant->name, "Orthodox Chess" ) || 
				!strcmp( selectedVariant->name, "Fisher Random Chess" ) || 
				!strcmp( selectedVariant->name, "Almost Chess" ) || 
				!strcmp( armyName, "Fabulous FIDEs" ) ||
				!strcmp( armyName, "Almost Chess army" ) )
			{
				AddPiece( OrthodoxRook::orthodoxRook, 1, 7, rook1File );
				AddPiece( OrthodoxRook::orthodoxRook, 1, 7, rook2File );
				AddPiece( OrthodoxKnight::orthodoxKnight, 1, 7, knight1File );
				AddPiece( OrthodoxKnight::orthodoxKnight, 1, 7, knight2File );
				AddPiece( OrthodoxBishop::orthodoxBishop, 1, 7, bishop1File );
				AddPiece( OrthodoxBishop::orthodoxBishop, 1, 7, bishop2File );
				if( armyName != NULL && !strcmp( armyName, "Almost Chess army" ) )
				{
					filename[20] = 'A';
					filename[21] = 'l';
					filename[22] = 'C';
					AddPiece( Chancellor::chancellor, 1, 7, queenFile );
				}
				else
					AddPiece( OrthodoxQueen::orthodoxQueen, 1, 7, queenFile );
			}
			else if( !strcmp( armyName, "Colorbound Clobberers" ) )
			{
				filename[20] = 'C';
				filename[21] = 'b';
				filename[22] = 'C';
				AddPiece( Bede::bede, 1, 7, rook1File );
				AddPiece( Bede::bede, 1, 7, rook2File );
				AddPiece( Waffle::waffle, 1, 7, knight1File );
				AddPiece( Waffle::waffle, 1, 7, knight2File );
				AddPiece( Fad::fad, 1, 7, bishop1File );
				AddPiece( Fad::fad, 1, 7, bishop2File );
				AddPiece( Cardinal::cardinal, 1, 7, queenFile );
				castlingPiecesSlideNorth[0] = false;
			}
			else if( !strcmp( armyName, "Colorbound Clobberers II" ) )
			{
				filename[20] = 'C';
				filename[21] = 'C';
				filename[22] = '2';
				AddPiece( Bede::bede, 1, 7, rook1File );
				AddPiece( Bede::bede, 1, 7, rook2File );
				AddPiece( Waffle::waffle, 1, 7, bishop1File );
				AddPiece( Waffle::waffle, 1, 7, bishop2File );
				AddPiece( Fad::fad, 1, 7, knight1File );
				AddPiece( Fad::fad, 1, 7, knight2File );
				AddPiece( Cardinal::cardinal, 1, 7, queenFile );
				castlingPiecesSlideNorth[0] = false;
			}
			else if( !strcmp( armyName, "Nutty Knights" ) )
			{
				filename[20] = 'N';
				filename[21] = 'K';
				filename[22] = 'n';
				AddPiece( ChargingRook::chargingRook, 1, 7, rook1File );
				AddPiece( ChargingRook::chargingRook, 1, 7, rook2File );
				AddPiece( Fibnif::fibnif, 1, 7, knight1File );
				AddPiece( Fibnif::fibnif, 1, 7, knight2File );
				AddPiece( ChargingKnight::chargingKnight, 1, 7, bishop1File );
				AddPiece( ChargingKnight::chargingKnight, 1, 7, bishop2File );
				AddPiece( Colonel::colonel, 1, 7, queenFile );
			}
			else if( !strcmp( armyName, "Remarkable Rookies" ) )
			{
				filename[20] = 'R';
				filename[21] = 'R';
				filename[22] = 'k';
				AddPiece( ShortRook::shortRook, 1, 7, rook1File );
				AddPiece( ShortRook::shortRook, 1, 7, rook2File );
				AddPiece( WoodyRook::woodyRook, 1, 7, knight1File );
				AddPiece( WoodyRook::woodyRook, 1, 7, knight2File );
				AddPiece( HalfDuck::halfDuck, 1, 7, bishop1File );
				AddPiece( HalfDuck::halfDuck, 1, 7, bishop2File );
				AddPiece( Chancellor::chancellor, 1, 7, queenFile );
			}
			else if( !strcmp( armyName, "Forward FIDEs" ) )
			{
				filename[20] = 'F';
				filename[21] = 'w';
				filename[22] = 'F';
				AddPiece( ChargingRook::chargingRook, 1, 7, rook1File );
				AddPiece( ChargingRook::chargingRook, 1, 7, rook2File );
				AddPiece( Knishop::knishop, 1, 7, knight1File );
				AddPiece( Knishop::knishop, 1, 7, knight2File );
				AddPiece( Bishight::bishight, 1, 7, bishop1File );
				AddPiece( Bishight::bishight, 1, 7, bishop2File );
				AddPiece( Forequeen::forequeen, 1, 7, queenFile );
			}
			else if( !strcmp( armyName, "All-Around Allstars" ) )
			{
				filename[20] = 'A';
				filename[21] = 'A';
				filename[22] = 's';
				AddPiece( ChargingRook::chargingRook, 1, 7, rook1File );
				AddPiece( ChargingRook::chargingRook, 1, 7, rook2File );
				AddPiece( Waffle::waffle, 1, 7, knight1File );
				AddPiece( Waffle::waffle, 1, 7, knight2File );
				AddPiece( OrthodoxBishop::orthodoxBishop, 1, 7, bishop1File );
				AddPiece( OrthodoxBishop::orthodoxBishop, 1, 7, bishop2File );
				AddPiece( Chancellor::chancellor, 1, 7, queenFile );
			}
			else if( !strcmp( armyName, "Meticulous Mashers" ) )
			{
				filename[20] = 'M';
				filename[21] = 'M';
				filename[22] = 'a';
				AddPiece( Fourfer::fourfer, 1, 7, rook1File );
				AddPiece( Fourfer::fourfer, 1, 7, rook2File );
				AddPiece( Fibnif::fibnif, 1, 7, knight1File );
				AddPiece( Fibnif::fibnif, 1, 7, knight2File );
				AddPiece( B4nD::b4nD, 1, 7, bishop1File );
				AddPiece( B4nD::b4nD, 1, 7, bishop2File );
				AddPiece( N2R4::n2R4, 1, 7, queenFile );
			}
			else if( !strcmp( armyName, "Andromedan Invaders" ) )
			{
				filename[20] = 'A';
				filename[21] = 'n';
				filename[22] = 'I';
				AddPiece( SV7::sv7, 1, 7, rook1File );
				AddPiece( SV7::sv7, 1, 7, rook2File );
				AddPiece( GreaterKnight::greaterKnight, 1, 7, knight1File );
				AddPiece( GreaterKnight::greaterKnight, 1, 7, knight2File );
				AddPiece( Cyborg::cyborg, 1, 7, bishop1File );
				AddPiece( Cyborg::cyborg, 1, 7, bishop2File );
				AddPiece( ShortQueen::shortQueen, 1, 7, queenFile );
			}
		}

		// *** ARMY ADJUSTMENTS *** //
		if( !strcmp( filename, "openings\\CWDA_FaF_v_RRk.txt" ) || 
			!strcmp( filename, "openings\\CWDA_RRk_v_FaF.txt" ) )
		{
			//	since the Remarkable Rookies have no diagonal sliders, 
			//	the Bishops are significantly more powerful, and the Queen 
			//	is a little more powerful
			OrthodoxBishop::orthodoxBishop.SetArmyAdjustment( 200 );
			OrthodoxQueen::orthodoxQueen.SetArmyAdjustment( 100 );
		}

		if( useOpeningBook )
		{
			Book *pBook = new Book( &board );
			board.SetOpeningBook( pBook, filename );
		}
	}

	LookupBoolParameter( "en-passant", enPassant );

	board.PostInitialize();
}

Piece *ChessGame::AddPiece
	( PieceType &pieceType,
	  int nPlayer,
	  int nRank,
	  int nFile )
{
	Piece *newPiece = Game::AddPiece( pieceType, nPlayer, nRank, nFile );

	//	if Pawn, set the promotion zone
	if( &pieceType == pawnPieceTypes[nPlayer] )
	{
		if( newPiece->GetPlayerNumber() == 0 )
			newPiece->GetPromotionZone().AddToZone( 56, 63 );
		else
			newPiece->GetPromotionZone().AddToZone( 0, 7 );
	}

	return newPiece;
}

void ChessGame::DeletePiece
	( Piece *piece )
{
}

PieceType **ChessGame::GetPieceTypesRequired
	( int &nPieceTypes )
{
	nPieceTypes = board.GetPieceTypeCount();
	return( board.GetPieceTypes() );
}

//void ChessGame::KPK()
//{
/**************************************************************************
 * Pawn is outside the square of the king 
 **************************************************************************/

/**************************************************************************
 *
 *  Friendly king is on same or adjacent file to the pawn, and the pawn is 
 *  on a file other than a rook file and ...
 *
 **************************************************************************/

/**************************************************************************
 *
 * a. friendly king is 2 ranks more advanced than the pawn 
 * b. friendly king is 1 rank more advanced than the pawn 
 *    i.  The friendly king is on the sixth rank.
 *    ii. The enemy king does not have direct opposition by being 2 ranks
 *        in front of the friendly king and on the same file.
 * c. friendly king is same rank as pawn
 *    i.  The enemy king is not 2-4 ranks more advanced that the pawn.
 *    ii. The pawn is on the sixth rank and the enemy king does not have
 *        direct opposition.
 * d. pawn is on the 7th rank, friendly king is on sixth rank and
 *    i.  The enemy king is not on the queening square.
 *    ii. The enemy is on the queening square but both kings are in the same
 *        file.
 * 
 **************************************************************************/
//}

void ChessGame::AddSpecialMoves
	( int currentPlayer, 
	  MovementList &stack, 
	  GameRec &gameRecord,
	  bool quiescentSearch )
{
	//	check for en-passant
	if( enPassant )
		Game::AddEnPassantMoves( stack, gameRecord, quiescentSearch );

	if( quiescentSearch || castlingType == CASTLING_TYPE_NONE )
		return;

	// *** CASTLING *** //

	if( castlingType == CASTLING_TYPE_STANDARD )
	{
		if( board.GetCurrentPlayerNumber() == 0 )
		{
			//	white to move
			Piece *king = board.GetKing( 0 );
			if( !king->HasMoved() )
			{
				//	king side
				if( board.GetSquareContents( 7 ) != NULL &&
					!board.GetSquareContents( 7 )->HasMoved() )
				{
					if( board.GetSquareContents( 5 ) == NULL &&
						board.GetSquareContents( 6 ) == NULL )
					{
						//	squares are vacant; make sure they are not attacked
						if( (!goalIsCheckmate && 
							 !king->IsCaptured()) || 
							(goalIsCheckmate && 
							 !IsSquareAttacked( 4, 1 ) &&
							 !IsSquareAttacked( 5, 1 ) && 
							 !IsSquareAttacked( 6, 1 )) )
						{
							Piece *castlingPiece = board.GetSquareContents( 7 );
							stack.BeginMoveAdd( Castling, 4, 6 );
							stack.AddPickUp( 4 );
							stack.AddPickUp( 7 );
							stack.AddDrop( king, 6 );
							stack.AddDrop( castlingPiece, 5 );
							stack.EndMoveAdd( 1000 );
						}
					}
				}
				//	queen side
				if( board.GetSquareContents( 0 ) != NULL &&
					!board.GetSquareContents( 0 )->HasMoved() &&
					!castlingPiecesAreColorbound[0] )
				{
					if( board.GetSquareContents( 3 ) == NULL &&
						board.GetSquareContents( 2 ) == NULL && 
						board.GetSquareContents( 1 ) == NULL )
					{
						//	square are vacant; make sure they are not attacked
						if( (!goalIsCheckmate && 
							 !king->IsCaptured()) || 
							(goalIsCheckmate && 
							 !IsSquareAttacked( 4, 1 ) && 
							 !IsSquareAttacked( 3, 1 ) && 
							 !IsSquareAttacked( 2, 1 )) )
						{
							Piece *castlingPiece = board.GetSquareContents( 0 );
							stack.BeginMoveAdd( Castling, 4, 2 );
							stack.AddPickUp( 4 );
							stack.AddPickUp( 0 );
							stack.AddDrop( king, 2 );
							stack.AddDrop( castlingPiece, 3 );
							stack.EndMoveAdd( 1000 );
						}
					}
				}
				else if( board.GetSquareContents( 0 ) != NULL &&
					!board.GetSquareContents( 0 )->HasMoved() &&
					castlingPiecesAreColorbound[0] )
				{
					//	this is a special case for Chess with Different Armies;
					//	colorbound rooks only move 2 squares when castling 
					//	queen-side, so that they don't change color
					if( board.GetSquareContents( 3 ) == NULL &&
						board.GetSquareContents( 2 ) == NULL && 
						board.GetSquareContents( 1 ) == NULL )
					{
						//	square are vacant; make sure they are not attacked
						if( (!goalIsCheckmate && 
							 !king->IsCaptured()) || 
							(goalIsCheckmate && 
							 !IsSquareAttacked( 4, 1 ) && 
							 !IsSquareAttacked( 3, 1 ) && 
							 !IsSquareAttacked( 2, 1 ) &&
							 !IsSquareAttacked( 1, 1 )) )
						{
							Piece *castlingPiece = board.GetSquareContents( 0 );
							stack.BeginMoveAdd( Castling, 4, 1 );
							stack.AddPickUp( 4 );
							stack.AddPickUp( 0 );
							stack.AddDrop( king, 1 );
							stack.AddDrop( castlingPiece, 2 );
							stack.EndMoveAdd( 1000 );
						}
					}
				}
			}
		}
		else
		{
			//	black to move
			Piece *king = board.GetKing( 1 );
			if( !king->HasMoved() )
			{
				//	king side
				if( board.GetSquareContents( 63 ) != NULL &&
					!board.GetSquareContents( 63 )->HasMoved() )
				{
					if( board.GetSquareContents( 61 ) == NULL &&
						board.GetSquareContents( 62 ) == NULL )
					{
						//	square are vacant; make sure they are not attacked
						if( (!goalIsCheckmate && 
							 !king->IsCaptured()) || 
							(goalIsCheckmate && 
							 !IsSquareAttacked( 60, 0 ) && 
							 !IsSquareAttacked( 61, 0 ) && 
							 !IsSquareAttacked( 62, 0 )) )
						{
							Piece *castlingPiece = board.GetSquareContents( 63 );
							stack.BeginMoveAdd( Castling, 60, 62 );
							stack.AddPickUp( 60 );
							stack.AddPickUp( 63 );
							stack.AddDrop( king, 62 );
							stack.AddDrop( castlingPiece, 61 );
							stack.EndMoveAdd( 1000 );
						}
					}
				}
				//	queen side
				if( board.GetSquareContents( 56 ) != NULL &&
					!board.GetSquareContents( 56 )->HasMoved() &&
					!castlingPiecesAreColorbound[1] )
				{
					if( board.GetSquareContents( 59 ) == NULL &&
						board.GetSquareContents( 58 ) == NULL && 
						board.GetSquareContents( 57 ) == NULL )
					{
						//	squares are vacant; make sure they are not attacked
						if( (!goalIsCheckmate && 
							 !king->IsCaptured()) || 
							(goalIsCheckmate && 
							 !IsSquareAttacked( 60, 0 ) && 
							 !IsSquareAttacked( 59, 0 ) && 
							 !IsSquareAttacked( 58, 0 )) )
						{
							Piece *castlingPiece = board.GetSquareContents( 56 );
							stack.BeginMoveAdd( Castling, 60, 58 );
							stack.AddPickUp( 60 );
							stack.AddPickUp( 56 );
							stack.AddDrop( king, 58 );
							stack.AddDrop( castlingPiece, 59 );
							stack.EndMoveAdd( 1000 );
						}
					}
				}
				else if( board.GetSquareContents( 56 ) != NULL &&
					!board.GetSquareContents( 56 )->HasMoved() &&
					castlingPiecesAreColorbound[1] )
				{
					//	this is a special case for Chess with Different Armies;
					//	colorbound rooks only move 2 squares when castling 
					//	queen-side, so that they don't change color
					if( board.GetSquareContents( 59 ) == NULL &&
						board.GetSquareContents( 58 ) == NULL && 
						board.GetSquareContents( 57 ) == NULL )
					{
						//	squares are vacant; make sure they are not attacked
						if( (!goalIsCheckmate && !king->IsCaptured()) || 
							(!IsSquareAttacked( 60, 0 ) && 
							 !IsSquareAttacked( 59, 0 ) && 
							 !IsSquareAttacked( 58, 0 ) &&
							 !IsSquareAttacked( 57, 0 )) )
						{
							Piece *castlingPiece = board.GetSquareContents( 56 );
							stack.BeginMoveAdd( Castling, 60, 57 );
							stack.AddPickUp( 60 );
							stack.AddPickUp( 56 );
							stack.AddDrop( king, 57 );
							stack.AddDrop( castlingPiece, 58 );
							stack.EndMoveAdd( 1000 );
						}
					}
				}
			}
		}
	}
	else if( castlingType == CASTLING_TYPE_FISCHER_RANDOM )
	{
		if( board.GetCurrentPlayerNumber() == 0 )
		{
			//	CASTLING in FISHER RANDOM CHESS - white to move
			Piece *king = board.GetKing( 0 );
			if( !king->HasMoved() )
			{
				//	a-side
				Piece *castlingPiece = board.GetSquareContents( 7 );

				if( castlingPiece != NULL &&
					!castlingPiece->HasMoved() )
				{
					//	see if all squares are vacant
					bool vacant = true;
					if( castlingPiece->GetSquareNumber() < D1 )
						for( int x = castlingPiece->GetSquareNumber() + 1; x <= D1 && vacant; x++ )
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
								//	piece in the way
								vacant = false;
					else if( castlingPiece->GetSquareNumber() > D1 )
						for( int x = castlingPiece->GetSquareNumber() - 1; x >= D1 && vacant; x-- )
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
								//	piece in the way
								vacant = false;
					if( king->GetSquareNumber() >= C1 )
						for( int x = king->GetSquareNumber(); x >= C1 && vacant; x-- )
						{
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
							{
								//	piece in the way
								vacant = false;
							}
							if( goalIsCheckmate && IsSquareAttacked( x, FLIP(king->GetPlayerNumber()) ) )
								//	square is attacked; can't do it
								vacant = false;
						}
					else
						for( int x = king->GetSquareNumber(); x <= C1 && vacant; x++ )
						{
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
							{
								//	piece in the way
								vacant = false;
							}
							if( goalIsCheckmate && IsSquareAttacked( x, FLIP(king->GetPlayerNumber()) ) )
								//	square is attacked; can't do it
								vacant = false;
						}
					if( vacant )
					{
						stack.BeginMoveAdd( Castling, king->GetSquareNumber(), C1 );
						stack.AddPickUp( king->GetSquareNumber() );
						stack.AddPickUp( castlingPiece->GetSquareNumber() );
						stack.AddDrop( king, C1 );
						stack.AddDrop( castlingPiece, D1 );
						stack.EndMoveAdd( 1000 );
					}
				}
				//	h-side
				castlingPiece = board.GetSquareContents( 0 );

				if( castlingPiece != NULL &&
					!castlingPiece->IsCaptured() &&
					!castlingPiece->HasMoved() )
				{
					//	see if all squares are vacant
					bool vacant = true;
					if( castlingPiece->GetSquareNumber() < F1 )
						for( int x = castlingPiece->GetSquareNumber() + 1; x <= F1 && vacant; x++ )
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
								//	piece in the way
								vacant = false;
					else if( castlingPiece->GetSquareNumber() > F1 )
						for( int x = castlingPiece->GetSquareNumber() - 1; x >= F1 && vacant; x-- )
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
								//	piece in the way
								vacant = false;
					if( king->GetSquareNumber() >= G1 )
						for( int x = king->GetSquareNumber(); x >= G1 && vacant; x-- )
						{
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
							{
								//	piece in the way
								vacant = false;
							}
							if( goalIsCheckmate && IsSquareAttacked( x, FLIP(king->GetPlayerNumber()) ) )
								//	square is attacked; can't do it
								vacant = false;
						}
					else
						for( int x = king->GetSquareNumber(); x <= G1 && vacant; x++ )
						{
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
							{
								//	piece in the way
								vacant = false;
							}
							if( goalIsCheckmate && IsSquareAttacked( x, FLIP(king->GetPlayerNumber()) ) )
								//	square is attacked; can't do it
								vacant = false;
						}
					if( vacant )
					{
						stack.BeginMoveAdd( Castling, king->GetSquareNumber(), G1 );
						stack.AddPickUp( king->GetSquareNumber() );
						stack.AddPickUp( castlingPiece->GetSquareNumber() );
						stack.AddDrop( king, G1 );
						stack.AddDrop( castlingPiece, F1 );
						stack.EndMoveAdd( 1000 );
					}
				}
			}
		}
		else
		{
			//	CASTLING in FISHER RANDOM CHESS - black to move
			Piece *king = board.GetKing( 1 );
			if( !king->HasMoved() )
			{
				//	a-side
				Piece *castlingPiece = board.GetSquareContents( 63 );

				if( castlingPiece != NULL &&
					!castlingPiece->IsCaptured() &&
					!castlingPiece->HasMoved() )
				{
					//	see if all squares are vacant
					bool vacant = true;
					if( castlingPiece->GetSquareNumber() < D8 )
						for( int x = castlingPiece->GetSquareNumber() + 1; x <= D8 && vacant; x++ )
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
								//	piece in the way
								vacant = false;
					else if( castlingPiece->GetSquareNumber() > D8 )
						for( int x = castlingPiece->GetSquareNumber() - 1; x >= D8 && vacant; x-- )
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
								//	piece in the way
								vacant = false;
					if( king->GetSquareNumber() >= C8 )
						for( int x = king->GetSquareNumber(); x >= C8 && vacant; x-- )
						{
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
							{
								//	piece in the way
								vacant = false;
							}
							if( goalIsCheckmate && IsSquareAttacked( x, FLIP(king->GetPlayerNumber()) ) )
								//	square is attacked; can't do it
								vacant = false;
						}
					else
						for( int x = king->GetSquareNumber(); x <= C8 && vacant; x++ )
						{
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
							{
								//	piece in the way
								vacant = false;
							}
							if( goalIsCheckmate && IsSquareAttacked( x, FLIP(king->GetPlayerNumber()) ) )
								//	square is attacked; can't do it
								vacant = false;
						}
					if( vacant )
					{
						stack.BeginMoveAdd( Castling, king->GetSquareNumber(), C8 );
						stack.AddPickUp( king->GetSquareNumber() );
						stack.AddPickUp( castlingPiece->GetSquareNumber() );
						stack.AddDrop( king, C8 );
						stack.AddDrop( castlingPiece, D8 );
						stack.EndMoveAdd( 1000 );
					}
				}
				//	h-side
				castlingPiece = board.GetSquareContents( 56 );

				if( castlingPiece != NULL &&
					!castlingPiece->IsCaptured() &&
					!castlingPiece->HasMoved() )
				{
					//	see if all squares are vacant
					bool vacant = true;
					if( castlingPiece->GetSquareNumber() < F8 )
						for( int x = castlingPiece->GetSquareNumber() + 1; x <= F8 && vacant; x++ )
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
								//	piece in the way
								vacant = false;
					else if( castlingPiece->GetSquareNumber() > F8 )
						for( int x = castlingPiece->GetSquareNumber() - 1; x >= F8 && vacant; x-- )
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
								//	piece in the way
								vacant = false;
					if( king->GetSquareNumber() >= G8 )
						for( int x = king->GetSquareNumber(); x >= G8 && vacant; x-- )
						{
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
							{
								//	piece in the way
								vacant = false;
							}
							if( goalIsCheckmate && IsSquareAttacked( x, FLIP(king->GetPlayerNumber()) ) )
								//	square is attacked; can't do it
								vacant = false;
						}
					else
						for( int x = king->GetSquareNumber(); x <= G8 && vacant; x++ )
						{
							if( board.GetSquareContents( x ) != NULL && 
								board.GetSquareContents( x ) != king && 
								board.GetSquareContents( x ) != castlingPiece )
							{
								//	piece in the way
								vacant = false;
							}
							if( goalIsCheckmate && IsSquareAttacked( x, FLIP(king->GetPlayerNumber()) ) )
								//	square is attacked; can't do it
								vacant = false;
						}
					if( vacant )
					{
						stack.BeginMoveAdd( Castling, king->GetSquareNumber(), G8 );
						stack.AddPickUp( king->GetSquareNumber() );
						stack.AddPickUp( castlingPiece->GetSquareNumber() );
						stack.AddDrop( king, G8 );
						stack.AddDrop( castlingPiece, F8 );
						stack.EndMoveAdd( 1000 );
					}
				}
			}
		}
	}
}

bool ChessGame::MoveBeingMade
	( MoveInfo &moveInfo,
	  GameRec &gameRecord )
{
	// *** UPDATE gameInt1 *** //

	//	gameInt1 is a 'user' field in the game records that a game can use
	//	for any purpose.  we will use it to track the forward progress of 
	//	the game, so we can determine which Phase we are in ('opening', 
	//	'midgame', or 'endgame' in our case)

	if( gameRecord.gameInt1 < 15 )
	{
		//	we are in the opening until gameInt1 reaches 15;
		//	in the opening, we increment gameInt1 for the following moves:
		//		- any capture
		//		- the first move of any piece
		//		- any pawn push to rank 5+ (code value of 4+)
		//		- in the special case of castling, add 4 to gameInt1
		if( moveInfo.type == Castling )
			gameRecord.gameInt1 += 4;
		else if( gameRecord.materialCaptured > 0 ||
			(gameRecord.pieceMoved->GetFlags() & (FLAGS_HAS_MOVED | FLAGS_HAS_MOVED_TWICE)) == FLAGS_HAS_MOVED ||
			(gameRecord.pieceMoved->IsPawn() && 
				board.ranks[gameRecord.pieceMoved->GetPlayerNumber()][gameRecord.pieceMoved->GetSquareNumber()] >= 4) )
			gameRecord.gameInt1++;
	}

	return true;
}

void ChessGame::GenerateMoves
	( int currentPlayer, 
	  MovementList &list )
{
	BitBoard64 pieces;

	// *** PAWNS *** //
	pieces = OrthodoxPawn::orthodoxPawn.GetPieces( currentPlayer );
	while( pieces )
	{
		int square = pieces.GetFirstBit();
		pieces.ClearBit( square );
		OrthodoxPawn::orthodoxPawn.BB64_GenerateDirectCaptures( list, currentPlayer, square );
		OrthodoxPawn::orthodoxPawn.BB64_GenerateDirectMovesNoCapture( list, currentPlayer, square );
	}
	//	handle the initial 2-step pawn move
	if( currentPlayer == 0 )
	{
		BitBoard64 secondRankPawns = player0PawnTwoSpaceZone & 
			OrthodoxPawn::orthodoxPawn.GetPieces( 0 );
		while( secondRankPawns )
		{
			int square = secondRankPawns.GetFirstBit();
			secondRankPawns.ClearBit( square );
			if( board.GetSquareContents( square + 8 ) == NULL && 
				board.GetSquareContents( square + 16 ) == NULL )
				//	squares are open; add the two-step move
				list.AddMove( square, square + 16 );
		}
	}
	else
	{
		BitBoard64 seventhRankPawns = player1PawnTwoSpaceZone & 
			OrthodoxPawn::orthodoxPawn.GetPieces( 1 );
		while( seventhRankPawns )
		{
			int square = seventhRankPawns.GetFirstBit();
			seventhRankPawns.ClearBit( square );
			if( board.GetSquareContents( square - 8 ) == NULL && 
				board.GetSquareContents( square - 16 ) == NULL )
				//	squares are open; add the two-step move
				list.AddMove( square, square - 16 );
		}
	}

	// *** KNIGHTS *** //
	pieces = OrthodoxKnight::orthodoxKnight.GetPieces( currentPlayer ) | 
		Chancellor::chancellor.GetPieces( currentPlayer ) |
		Cardinal::cardinal.GetPieces( currentPlayer );
	while( pieces )
	{
		int square = pieces.GetFirstBit();
		pieces.ClearBit( square );
		OrthodoxKnight::orthodoxKnight.BB64_GenerateDirectMoves( list, currentPlayer, square );
	}

	// *** KING *** //
	Piece *king = board.GetKing( currentPlayer );
	OrthodoxKing::orthodoxKing.BB64_GenerateDirectMoves( list, currentPlayer, king->GetSquareNumber() );

	// *** ROOK sliders *** //
	BitBoard64 friends = board.bb64_friends[currentPlayer];
	pieces = OrthodoxRook::orthodoxRook.GetPieces( currentPlayer ) |  
		OrthodoxQueen::orthodoxQueen.GetPieces( currentPlayer ) | 
		Chancellor::chancellor.GetPieces( currentPlayer );
	while( pieces )
	{
		int square = pieces.GetFirstBit();
		pieces.ToggleBit( square );

		BitBoard64 moves;
		moves = board.bb64_rook00attack[square * board.rook00attackWidth + 
			((board.bb64_blocker >> board.shift00[square]) & board.mask00[square])] | 
			board.bb64_rook90attack[square * board.rook90attackWidth + 
			((board.bb64_blocker90 >> board.shift90[square]) & board.mask90[square])];
		moves = moves & (~friends);
		if( moves )
			list.AddBitBoard64Moves( square, moves );
	}

	// *** BISHOP sliders *** // 
	pieces = OrthodoxBishop::orthodoxBishop.GetPieces( currentPlayer ) | 
		OrthodoxQueen::orthodoxQueen.GetPieces( currentPlayer ) |
		Cardinal::cardinal.GetPieces( currentPlayer );
	while( pieces )
	{
		int square = pieces.GetFirstBit();
		pieces.ToggleBit( square );

		BitBoard64 moves;
		moves = board.bb64_bishop45attack[square * board.bishop45attackWidth + 
			((board.bb64_blocker45 >> board.shift45[square]) & board.mask45[square])] | 
			board.bb64_bishop135attack[square * board.bishop135attackWidth + 
			((board.bb64_blocker135 >> board.shift135[square]) & board.mask135[square])];
		moves = moves & (~friends);
		if( moves )
			list.AddBitBoard64Moves( square, moves );
	}

	//	check for en-passant
	AddSpecialMoves( currentPlayer, list, board.GetCurrentGameRecord(), false );
}

void ChessGame::GenerateCaptures
	( int currentPlayer, 
	  MovementList &list )
{
	BitBoard64 pieces;

	// *** PAWNS *** //
	pieces = OrthodoxPawn::orthodoxPawn.GetPieces( currentPlayer );
	while( pieces )
	{
		int square = pieces.GetFirstBit();
		pieces.ClearBit( square );
		OrthodoxPawn::orthodoxPawn.BB64_GenerateDirectCaptures( list, currentPlayer, square );
	}

	// *** KNIGHTS *** //
	pieces = OrthodoxKnight::orthodoxKnight.GetPieces( currentPlayer ) | 
		Chancellor::chancellor.GetPieces( currentPlayer ) |
		Cardinal::cardinal.GetPieces( currentPlayer );
	while( pieces )
	{
		int square = pieces.GetFirstBit();
		pieces.ClearBit( square );
		OrthodoxKnight::orthodoxKnight.BB64_GenerateDirectCaptures( list, currentPlayer, square );
	}

	// *** KING *** //
	Piece *king = board.GetKing( currentPlayer );
	OrthodoxKing::orthodoxKing.BB64_GenerateDirectCaptures( list, currentPlayer, king->GetSquareNumber() );

	// *** ROOK sliders *** //
	BitBoard64 enemies = board.bb64_friends[FLIP(currentPlayer)];
	pieces = OrthodoxRook::orthodoxRook.GetPieces( currentPlayer ) |  
		OrthodoxQueen::orthodoxQueen.GetPieces( currentPlayer ) | 
		Chancellor::chancellor.GetPieces( currentPlayer );
	while( pieces )
	{
		int square = pieces.GetFirstBit();
		pieces.ToggleBit( square );

		BitBoard64 moves;
		moves = board.bb64_rook00attack[square * board.rook00attackWidth + 
			((board.bb64_blocker >> board.shift00[square]) & board.mask00[square])] | 
			board.bb64_rook90attack[square * board.rook90attackWidth + 
			((board.bb64_blocker90 >> board.shift90[square]) & board.mask90[square])];
		moves = moves & enemies;
		if( moves )
			list.AddBitBoard64Captures( square, moves );
	}

	// *** BISHOP sliders *** // 
	pieces = OrthodoxBishop::orthodoxBishop.GetPieces( currentPlayer ) | 
		OrthodoxQueen::orthodoxQueen.GetPieces( currentPlayer ) |
		Cardinal::cardinal.GetPieces( currentPlayer );
	while( pieces )
	{
		int square = pieces.GetFirstBit();
		pieces.ToggleBit( square );

		BitBoard64 moves;
		moves = board.bb64_bishop45attack[square * board.bishop45attackWidth + 
			((board.bb64_blocker45 >> board.shift45[square]) & board.mask45[square])] | 
			board.bb64_bishop135attack[square * board.bishop135attackWidth + 
			((board.bb64_blocker135 >> board.shift135[square]) & board.mask135[square])];
		moves = moves & enemies;
		if( moves )
			list.AddBitBoard64Captures( square, moves );
	}

	//	check for en-passant
	if( enPassant )
		Game::AddEnPassantMoves( list, board.GetCurrentGameRecord(), true );
}

bool ChessGame::IsSquareAttacked
	( int squareNumber, 
	  int playerNumber )
{
	BitBoard64 pieces;
	int otherPlayer = FLIP(playerNumber);

	// *** PAWNS *** //
	if( OrthodoxPawn::orthodoxPawn.bb64_attack[otherPlayer][squareNumber] &
		OrthodoxPawn::orthodoxPawn.GetPieces( playerNumber ) )
		return true;

	// *** KNIGHTS *** //
	if( OrthodoxKnight::orthodoxKnight.bb64_attack[otherPlayer][squareNumber] &
		(OrthodoxKnight::orthodoxKnight.GetPieces( playerNumber ) | 
		 Chancellor::chancellor.GetPieces( playerNumber ) | 
		 Cardinal::cardinal.GetPieces( playerNumber )) )
		return true;

	// *** KINGS *** //
	if( OrthodoxKing::orthodoxKing.bb64_attack[otherPlayer][squareNumber] &
		OrthodoxKing::orthodoxKing.GetPieces( playerNumber ) )
		return true;

	// *** ROOKS *** //
	pieces = OrthodoxRook::orthodoxRook.bb64_attack[otherPlayer][squareNumber] & 
		(OrthodoxRook::orthodoxRook.GetPieces( playerNumber ) | 
		 OrthodoxQueen::orthodoxQueen.GetPieces( playerNumber ) | 
		 Chancellor::chancellor.GetPieces( playerNumber ));
	while( pieces )
	{
		int square = pieces.GetFirstBit();
		if( ! ((bool) (board.bb64_rays[(squareNumber * board.nSquares) + square] & 
			  board.bb64_blocker & ~BitBoard64::GetPositionBitBoard( square ))) )
			return true;
		pieces.ClearBit( square );
	}

	// *** BISHOPS *** //
	pieces = OrthodoxBishop::orthodoxBishop.bb64_attack[otherPlayer][squareNumber] & 
		(OrthodoxBishop::orthodoxBishop.GetPieces( playerNumber ) | 
		 OrthodoxQueen::orthodoxQueen.GetPieces( playerNumber ) | 
		 Cardinal::cardinal.GetPieces( playerNumber ));
	while( pieces )
	{
		int square = pieces.GetFirstBit();
		if( ! ((bool) (board.bb64_rays[(squareNumber * board.nSquares) + square] & 
			  board.bb64_blocker & ~BitBoard64::GetPositionBitBoard( square ))) )
			return true;
		pieces.ClearBit( square );
	}

	return false;
}

Phase &ChessGame::AdjustEvaluation
	( int &eval,
	  PawnHash *pPawnHash )
{
	Phase *currentPhase = phases;

	if( board.GetCurrentGameRecord().gameInt1 < 15 )
	{
		// *** The OPENING *** //
		int factor = MAX(board.GetCurrentGameRecord().gameInt1 - 10, 0) * 2;
		int factor5 = 320 - (factor << 5);  //    \_  different adjustments
		int factor4 = 160 - (factor << 4);  //    /   from large to small
		int factor3 = 80 - (factor << 3);   //   / 

		if( board.GetKing( 0 )->GetFlags() & FLAGS_HAS_CASTLED )
			eval += factor5;
		else
		{
			if( board.GetKing( 0 )->HasMoved() || 
				((board.GetSquareContents( 7 ) == NULL ||
				  board.GetSquareContents( 7 )->HasMoved()) &&
				 (board.GetSquareContents( 0 ) == NULL ||
				  board.GetSquareContents( 0 )->HasMoved())) )
				eval -= factor4;
		}
		if( board.GetKing( 1 )->GetFlags() & FLAGS_HAS_CASTLED )
			eval -= factor5;
		else
		{
			if( board.GetKing( 1 )->HasMoved() || 
				((board.GetSquareContents( 63 ) == NULL ||
				  board.GetSquareContents( 63 )->HasMoved()) &&
				 (board.GetSquareContents( 56 ) == NULL ||
				  board.GetSquareContents( 56 )->HasMoved())) )
				eval += factor4;
		}

		//	penalize queen for moving; 
		//	in opening we assume only one queen
		BitBoard64 bb;
		bb = queenPieceTypes[0]->GetPieces( 0 );
		if( bb )
		{
			int square = bb.GetFirstBit();
			if( board.GetSquareContents( square )->HasMoved() )
				eval -= factor5;
		}
		bb = queenPieceTypes[1]->GetPieces( 1 );
		if( bb )
		{
			int square = bb.GetFirstBit();
			if( board.GetSquareContents( square )->HasMoved() )
				eval += factor5;
		}

		if( testingPosition )
			eval += PenalizePiecesForMovingMultipleTimes();

		//	don't block the king or queen's pawns in the opening!!!
		if( board.GetSquareContents( D3 ) != NULL &&
			board.GetSquareContents( D2 ) != NULL &&
			!board.GetSquareContents( D2 )->HasMoved() )
			eval -= factor5;
		if( board.GetSquareContents( E3 ) != NULL &&
			board.GetSquareContents( E2 ) != NULL &&
			!board.GetSquareContents( E2 )->HasMoved() )
			eval -= factor5;
		if( board.GetSquareContents( D6 ) != NULL &&
			board.GetSquareContents( D7 ) != NULL &&
			!board.GetSquareContents( D7 )->HasMoved() )
			eval += factor5;
		if( board.GetSquareContents( E6 ) != NULL &&
			board.GetSquareContents( E7 ) != NULL &&
			!board.GetSquareContents( E7 )->HasMoved() )
			eval += factor5;
	}
	else if( board.GetNumberOfCapturedPieces() > 24 ||
		board.GetMaterial( 0 ) - board.GetPawnMaterial( 0 ) + 
		board.GetMaterial( 1 ) - board.GetPawnMaterial( 1 ) < 18000 )
		//	we are in the end-game
		currentPhase = phases + 2;
	else
		//	we are in the mid-game
		currentPhase = phases + 1;


	// *** OUTPOSTS *** //

	//	outposts are minor pieces in central squares that cannot be driven off by enemy pawns.
	//	the basic output bonus is the value of the piece's square in the 'outpost' array, times
	//	the value of the piece type's 'outpost factor' (stored in 'gameInt1'.)

	//	a posted piece that is protected by one of its own pawns is extra good, so
	//	it gets 150% of the above bonus.  a posted piece protected by two of its
	//	own pawns is really, really good, because if the posted piece is captured,
	//	the player will have a protected, passed pawn.  a posted piece that is thus
	//	protected by 2 pawns gets double the standard output bonus.

	if( usePawnStructureEvaluation )
	{
		BitBoard64 bb64 = minorPieceTypes[0][0]->GetPieces( 0 ) | 
						minorPieceTypes[0][1]->GetPieces( 0 ) | 
						minorPieceTypes[0][2]->GetPieces( 0 ) | 
						minorPieceTypes[0][3]->GetPieces( 0 );
		//	loop through all player 0's minor pieces
		while( bb64 )
		{
			int square = bb64.GetFirstBit();
			bb64.ToggleBit( square );
			Piece *minorPiece = board.GetSquareContents( square );
			if( outpost[square] > 0 && minorPiece->GetType().gameInt1 > 0 &&
				((int) (pPawnHash->nPawnsPerFile[1][board.files[0][square]-1]) == 0 ||
				(int) (pPawnHash->backPawnRank[1][board.files[0][square]-1]) <= board.ranks[0][minorPiece->GetSquareNumber()]) &&
				((int) (pPawnHash->nPawnsPerFile[1][board.files[0][square]+1]) == 0 ||
				(int) (pPawnHash->backPawnRank[1][board.files[0][square]+1]) <= board.ranks[0][minorPiece->GetSquareNumber()]) )
			{
				//	basic bonus for player 0's posted piece
				eval += outpost[square] * minorPiece->GetType().gameInt1;
				int pawnSquare1 = board.GetMovementMatrix( DIRECTION_SE )[square];
				int pawnSquare2 = board.GetMovementMatrix( DIRECTION_SW )[square];
				if( board.GetSquareContents( pawnSquare1 ) != NULL &&
					board.GetSquareContents( pawnSquare1 )->GetPlayerNumber() == 0 && 
					board.GetSquareContents( pawnSquare1 )->GetType().IsPawn() )
					//	additional 50% bonus for this pawn defender
					eval += (outpost[square] * minorPiece->GetType().gameInt1) >> 1;
				if( board.GetSquareContents( pawnSquare2 ) != NULL &&
					board.GetSquareContents( pawnSquare2 )->GetPlayerNumber() == 0 && 
					board.GetSquareContents( pawnSquare2 )->GetType().IsPawn() )
					//	additional 50% bonus for this pawn defender
					eval += (outpost[square] * minorPiece->GetType().gameInt1) >> 1;
			}
		}
		bb64 = minorPieceTypes[1][0]->GetPieces( 1 ) | 
			minorPieceTypes[1][1]->GetPieces( 1 ) | 
			minorPieceTypes[1][2]->GetPieces( 1 ) | 
			minorPieceTypes[1][3]->GetPieces( 1 );
		//	loop through all player 1's minor pieces
		while( bb64 )
		{
			int square = bb64.GetFirstBit();
			bb64.ToggleBit( square );
			Piece *minorPiece = board.GetSquareContents( square );
			if( minorPiece != NULL &&
				!minorPiece->IsCaptured() &&
				outpost[board.flipSquare[1][square]] > 0 && minorPiece->GetType().gameInt1 > 0 &&
				((int) (pPawnHash->nPawnsPerFile[0][board.files[0][square]-1]) == 0 ||
				(int) (pPawnHash->backPawnRank[0][board.files[0][square]-1]) >= board.ranks[0][minorPiece->GetSquareNumber()]) &&
				((int) (pPawnHash->nPawnsPerFile[0][board.files[0][square]+1]) == 0 ||
				(int) (pPawnHash->backPawnRank[0][board.files[0][square]+1]) >= board.ranks[0][minorPiece->GetSquareNumber()]) )
			{
				//	basic bonus for player 1's posted piece
				eval -= outpost[board.flipSquare[1][square]] *
						minorPiece->GetType().gameInt1;
				int pawnSquare1 = board.GetMovementMatrix( DIRECTION_NE )[square];
				if( board.GetSquareContents( pawnSquare1 ) != NULL &&
					board.GetSquareContents( pawnSquare1 )->GetPlayerNumber() == 1 && 
					board.GetSquareContents( pawnSquare1 )->GetType().IsPawn() )
					//	additional 50% bonus for this pawn defender
					eval -= (outpost[board.flipSquare[1][square]] *
							minorPiece->GetType().gameInt1) >> 1;
				int pawnSquare2 = board.GetMovementMatrix( DIRECTION_NW )[square];
				if( board.GetSquareContents( pawnSquare2 ) != NULL &&
					board.GetSquareContents( pawnSquare2 )->GetPlayerNumber() == 1 && 
					board.GetSquareContents( pawnSquare2 )->GetType().IsPawn() )
					//	additional 50% bonus for this pawn defender
					eval -= (outpost[board.flipSquare[1][square]] *
							minorPiece->GetType().gameInt1) >> 1;
			}
		}
	}

	// *** ROOKS (and other north-sliders) ON OPEN and SEMI-OPEN FILE BONUS *** //

	if( castlingPiecesSlideNorth[0] && pPawnHash != NULL )
	{
		BitBoard64 castlingPieces = castlingPieceTypes[0]->GetPieces( 0 );
		while( castlingPieces )
		{
			int square = castlingPieces.GetFirstBit();
			castlingPieces.ToggleBit( square );
			if( pPawnHash->nPawnsPerFile[0][board.files[0][square]] == 0 )
			{
				//	file at least semi-open
				eval += 50;
				if( pPawnHash->nPawnsPerFile[1][board.files[0][square]] == 0 )
					//	fully open file
					eval += 20;
			}
		}
	}
	if( castlingPiecesSlideNorth[1] && pPawnHash != NULL )
	{
		BitBoard64 castlingPieces = castlingPieceTypes[1]->GetPieces( 1 );
		while( castlingPieces )
		{
			int square = castlingPieces.GetFirstBit();
			castlingPieces.ToggleBit( square );
			if( pPawnHash->nPawnsPerFile[1][board.files[0][square]] == 0 )
			{
				//	file at least semi-open
				eval -= 50;
				if( pPawnHash->nPawnsPerFile[0][board.files[0][square]] == 0 )
					//	fully open file
					eval -= 20;
			}
		}
	}

	//	TWO-BISHOPS BONUS
	//		TODO: temporarily removed

	//	try to keep our queens, for their viscious attack value! (ASYMMETRY)
	//		TODO: temporarily removed

	//	Fianchetto bishop bonus (given to Fads and Half-Ducks too)
	if( board.GetKing( 0 )->GetSquareNumber() >= F1 &&
		board.GetKing( 0 )->GetSquareNumber() <= H1 &&
		board.GetSquareContents( G2 ) != NULL &&
		(board.GetSquareContents( G2 )->GetType() == OrthodoxBishop::orthodoxBishop ||
		 board.GetSquareContents( G2 )->GetType() == Fad::fad ||
		 board.GetSquareContents( G2 )->GetType() == HalfDuck::halfDuck) )
		eval += 80;
	else if( board.GetKing( 0 )->GetSquareNumber() >= A1 &&
		board.GetKing( 0 )->GetSquareNumber() <= C1 &&
		board.GetSquareContents( B2 ) != NULL &&
		(board.GetSquareContents( B2 )->GetType() == OrthodoxBishop::orthodoxBishop ||
		 board.GetSquareContents( B2 )->GetType() == Fad::fad ||
		 board.GetSquareContents( B2 )->GetType() == HalfDuck::halfDuck) )
		eval += 80;
	if( board.GetKing( 0 )->GetSquareNumber() >= F8 &&
		board.GetKing( 0 )->GetSquareNumber() <= H8 &&
		board.GetSquareContents( G7 ) != NULL &&
		(board.GetSquareContents( G7 )->GetType() == OrthodoxBishop::orthodoxBishop ||
		 board.GetSquareContents( G7 )->GetType() == Fad::fad ||
		 board.GetSquareContents( G7 )->GetType() == HalfDuck::halfDuck) )
		eval -= 80;
	else if( board.GetKing( 0 )->GetSquareNumber() >= A8 &&
		board.GetKing( 0 )->GetSquareNumber() <= C8 &&
		board.GetSquareContents( B7 ) != NULL &&
		(board.GetSquareContents( B7 )->GetType() == OrthodoxBishop::orthodoxBishop ||
		 board.GetSquareContents( B7 )->GetType() == Fad::fad ||
		 board.GetSquareContents( B7 )->GetType() == HalfDuck::halfDuck) )
		eval -= 80;

	return *currentPhase;
}

int ChessGame::EnumeratePromotions
	( Piece *piece,
	  int fromSquare,
	  int toSquare,
	  PieceType **promotions )
{
	int count = 0;
	for( int i = 0; i < board.GetPieceTypeCount(); i++ )
	{
		PieceType *pieceType = board.GetPieceTypes()[i];
		if( !pieceType->IsPawn() &&
			!pieceType->IsRoyal() )
		{
			promotions[count++] = pieceType;
		}
	}
	return count;
}


word32 ChessGame::AdjustPrimaryHash
	( word32 primaryHash )
{
	//	check for castling privledges
	if( !board.GetKing( 0 )->HasMoved() )
	{
		Piece *castlingPiece = board.GetSquareContents( A1 );
		if( castlingPiece != NULL && 
			!castlingPiece->HasMoved() && 
			&(castlingPiece->GetType()) == castlingPieceTypes[0] )
			primaryHash ^= castlingHash0q;
		castlingPiece = board.GetSquareContents( H1 );
		if( castlingPiece != NULL && 
			!castlingPiece->HasMoved() && 
			&(castlingPiece->GetType()) == castlingPieceTypes[0] )
			primaryHash ^= castlingHash0k;
	}
	else if( !board.GetKing( 1 )->HasMoved() )
	{
		Piece *castlingPiece = board.GetSquareContents( A8 );
		if( castlingPiece != NULL && 
			!castlingPiece->HasMoved() && 
			&(castlingPiece->GetType()) == castlingPieceTypes[1] )
			primaryHash ^= castlingHash1q;
		castlingPiece = board.GetSquareContents( H8 );
		if( castlingPiece != NULL && 
			!castlingPiece->HasMoved() && 
			&(castlingPiece->GetType()) == castlingPieceTypes[1] )
			primaryHash ^= castlingHash1k;
	}

	if( enPassant )
		return primaryHash ^ GetEnPassantHash( board.GetGameRecord( board.GetGamePly() - 1 ), enPassantHashMap );
	return primaryHash;
}

void ChessGame::AboutToStartThinking
	( int currentPlayer )
{
}

