
/***************************************************************************

                                 ChessV

                   COPYRIGHT (C) 2007 BY GREGORY STRONG

This file is part of ChessV.  ChessV is free software; you can redistribute
it and/or modify it under the terms of the GNU General Public License as 
published by the Free Software Foundation; either version 2 of the License, 
or (at your option) any later version.

ChessV is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
more details; the file 'COPYING' contains the License text, but if for
some reason you need a copy, please write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

****************************************************************************/


#include "StdAfx.h"
#include "../../ChessV.h"
#include "../../PieceType.h"
#include "../../Piece.h"
#include "../../Direction.h"
#include "../../GameParameters.h"
#include "ModernShatranjGame.h"


// *** DEBUG MEMORY ALLOCATION *** //
#ifdef _DEBUG
#define new SAFE_NEW
#endif


Game *CreateModernShatranjGame( Board &brd, Player &plr0, Player &plr1 )
{
	return new ModernShatranjGame( brd, plr0, plr1 );
}


ModernShatranjGame::ModernShatranjGame( Board &board, Player &whitePlayer, Player &blackPlayer ):
	ChessGame(board, whitePlayer, blackPlayer)
{
}

void ModernShatranjGame::AddPlayerPieceTypes( char *gameName )
{
	// *** MODERN SHATRANJ *** //
	if( !strcmp( gameName, "Modern Shatranj" ) )
	{
		//	add the piece types that each player may have
		board.AddPlayerPieceTypeBothPlayers( OrthodoxRook::orthodoxRook );
		board.AddPlayerPieceTypeBothPlayers( ElephantFerz::elephantFerz );
		board.AddPlayerPieceTypeBothPlayers( OrthodoxKnight::orthodoxKnight );
		board.AddPlayerPieceTypeBothPlayers( General::general );
		board.AddPlayerPieceTypeBothPlayers( OrthodoxKing::orthodoxKing );
		board.AddPlayerPieceTypeBothPlayers( LamePawn::lamePawn );
		//	add piece type references to the following arrays, 
		//	minorPieceTypes, etc., used for giving bonuses and 
		//	penalties in positional evaluation
		castlingPieceTypes[0] = &OrthodoxRook::orthodoxRook;
		castlingPieceTypes[1] = &OrthodoxRook::orthodoxRook;
		minorPieceTypes[0][0] = &OrthodoxKnight::orthodoxKnight;
		minorPieceTypes[1][0] = &OrthodoxKnight::orthodoxKnight;
		minorPieceTypes[0][1] = &ElephantFerz::elephantFerz;
		minorPieceTypes[1][1] = &ElephantFerz::elephantFerz;
		minorPieceTypes[0][2] = &General::general;
		minorPieceTypes[1][2] = &General::general;
		minorPieceTypes[0][3] = &NullPieceType64::nullPieceType64;
		minorPieceTypes[1][3] = &NullPieceType64::nullPieceType64;
		colorboundPieceTypes[0][0] = &Elephant::elephant;
		colorboundPieceTypes[1][0] = &Elephant::elephant;
		colorboundPieceTypes[0][1] = &NullPieceType64::nullPieceType64;
		colorboundPieceTypes[1][1] = &NullPieceType64::nullPieceType64;
		queenPieceTypes[0] = &NullPieceType64::nullPieceType64;
		queenPieceTypes[1] = &NullPieceType64::nullPieceType64;
		pawnPieceTypes[0] = &LamePawn::lamePawn;
		pawnPieceTypes[1] = &LamePawn::lamePawn;
		castlingPiecesSlideNorth[0] = true;
		castlingPiecesSlideNorth[1] = true;
	}
}

void ModernShatranjGame::ChangeRulesByVariant
	( char *gameName,
	  char *&array,
	  char *&book )
{
	// *** MODERN SHATRANJ *** //
	if( !strcmp( gameName, "Modern Shatranj" ) )
	{
		//	no castling in Modern Shatranj
		castlingType = CASTLING_TYPE_NONE;
		//	no en-Passant
		enPassant = false;
		//	use 'bare king' rule
		bareKingRule = true;
		//	stalemate is a win
		stalemateResult = INFINITY;
		//	change pawn promotion
		LamePawn::lamePawn.SetPromotionType( PromoteByReplacementOrToVariableTypes );
		//	Knights and Rooks are move valuble than usual
		OrthodoxKnight::orthodoxKnight.SetBaseValue( 4000 );
		OrthodoxRook::orthodoxRook.SetBaseValue( 5500 );
		ElephantFerz::elephantFerz.SetBaseValue( 3000 );
		//	place pieces
		array = "rnekgenr/pppppppp/8/8/8/8/PPPPPPPP/RNEKGENR";
		//	set name of opening book
		book = "openings\\ModernShatranj.txt";
	}
}

bool ModernShatranjGame::MoveBeingMade
	( MoveInfo &moveInfo,
	  GameRec &gameRecord )
{
	// *** UPDATE gameInt2 *** //

	//	the gameInt2 user field is used in this game to keep track 
	//	of which pieces have been promoted by replacement.  this game 
	//	has a special rule that a player can only promote by replacement 
	//	to an elephant once in a game, to a rook once in a game, and to	
	//	a knight once in a game.

	#define WHITE_ROOK_PROMOTION		 1
	#define WHITE_KNIGHT_PROMOTION		 2
	#define WHITE_ELEPHANT_PROMOTION	 4
	#define BLACK_ROOK_PROMOTION		 8
	#define BLACK_KNIGHT_PROMOTION		16
	#define BLACK_ELEPHANT_PROMOTION	32

	if( moveInfo.type == Replacement || 
		moveInfo.type == CaptureWithReplacement )
	{
		if( moveInfo.pPieceMoved->GetPlayerNumber() == 0 )
		{
			if( (moveInfo.promotion) == &OrthodoxRook::orthodoxRook )
			{
				if( gameRecord.gameInt2 & WHITE_ROOK_PROMOTION )
					//	already promoted to a rook once, therefore move is illegal 
					return false;
				gameRecord.gameInt2 |= WHITE_ROOK_PROMOTION;
			}
			else if( (moveInfo.promotion) == &OrthodoxKnight::orthodoxKnight )
			{
				if( gameRecord.gameInt2 & WHITE_KNIGHT_PROMOTION )
					//	already promoted to a knight once, therefore move is illegal 
					return false;
				gameRecord.gameInt2 |= WHITE_KNIGHT_PROMOTION;
			}
			else if( (moveInfo.promotion) == &ElephantFerz::elephantFerz )
			{
				if( gameRecord.gameInt2 & WHITE_ELEPHANT_PROMOTION )
					//	already promoted to an elephant once, therefore move is illegal 
					return false;
				//	make sure this doesn't result in another Elephant on the same color
				if( (ElephantFerz::elephantFerz.GetPieces( 0 ) &
					whiteSquares).GetBitCount() > 1 )
					return false;
				if( (ElephantFerz::elephantFerz.GetPieces( 0 ) &
					blackSquares).GetBitCount() > 1 )
					return false;
				gameRecord.gameInt2 |= WHITE_ELEPHANT_PROMOTION;
			}
		}
		else
		{
			if( (moveInfo.promotion) == &OrthodoxRook::orthodoxRook )
			{
				if( gameRecord.gameInt2 & BLACK_ROOK_PROMOTION )
					//	already promoted to a rook once, therefore move is illegal 
					return false;
				gameRecord.gameInt2 |= BLACK_ROOK_PROMOTION;
			}
			else if( (moveInfo.promotion) == &OrthodoxKnight::orthodoxKnight )
			{
				if( gameRecord.gameInt2 & BLACK_KNIGHT_PROMOTION )
					//	already promoted to a knight once, therefore move is illegal 
					return false;
				gameRecord.gameInt2 |= BLACK_KNIGHT_PROMOTION;
			}
			else if( (moveInfo.promotion) == &ElephantFerz::elephantFerz )
			{
				if( gameRecord.gameInt2 & BLACK_ELEPHANT_PROMOTION )
					//	already promoted to an elephant once, therefore move is illegal 
					return false;
				//	make sure this doesn't result in another Elephant on the same color
				if( (ElephantFerz::elephantFerz.GetPieces( 1 ) &
					whiteSquares).GetBitCount() > 1 )
					return false;
				if( (ElephantFerz::elephantFerz.GetPieces( 1 ) &
					blackSquares).GetBitCount() > 1 )
					return false;
				gameRecord.gameInt2 |= BLACK_ELEPHANT_PROMOTION;
			}
		}
	}

	return ChessGame::MoveBeingMade( moveInfo, gameRecord );
}

int ModernShatranjGame::EnumeratePromotions
	( Piece *piece, 
	  int fromSquare,
	  int toSquare,
	  PieceType **promotions,
	  bool quiescentSearch )
{
	int count = 1;

	promotions[0] = &(General::general);

	return count;
}

int ModernShatranjGame::EnumeratePromotions
	( Piece *piece, 
	  int fromSquare,
	  int toSquare,
	  Piece **pieces,
	  bool quiescentSearch )
{
	int count = 0;

	bool rookFound = false;
	bool knightFound = false;
	bool elephantFound = false;

	for( int x = 0; x < board.GetNumberOfPieces( piece->GetPlayerNumber() ); x++ )
	{
		Piece *currentPiece = board.GetPiece( piece->GetPlayerNumber(), x );
		if( currentPiece->IsCaptured() )
		{
			if( &(currentPiece->GetType()) == &(OrthodoxRook::orthodoxRook) && !rookFound )
			{
				rookFound = true;
				pieces[count++] = currentPiece;
			}
			else if( &(currentPiece->GetType()) == &(OrthodoxKnight::orthodoxKnight) && !knightFound )
			{
				knightFound = true;
				pieces[count++] = currentPiece;
			}
			else if( &(currentPiece->GetType()) == &(ElephantFerz::elephantFerz) && !elephantFound )
			{
				elephantFound = true;
				pieces[count++] = currentPiece;
			}
		}
	}

	return count;
}
